@@ -34,7 +34,18 @@ public:
 	Context(const Context&) = delete;
 	Context() = default;
 
-	void Init(int width, int height, const std::string& name);
+	void Init(int width, int height, const std::string& name,
+			bool resizable,
+			bool visible,
+			bool decorated,
+			bool focused,
+			bool auto_iconify,
+			bool floating,
+			bool maximized,
+			bool transperent,
+			bool focus_on_show,
+			bool scale_to_monitor,
+			bool full_screen);
 
 	void Resize(int width, int height);
 
@@ -52,6 +63,9 @@ public:
 
 	std::array<float, 4> clearColor = {0.1f, 0.1f, 0.1f, 1.0f};
 
+	py::function mouse_button_callback;
+	py::function mouse_position_callback;
+	py::function keyboard_callback;
 private:
 	GLFWwindow* m_window = nullptr;
 	int m_width;
@@ -62,7 +76,18 @@ private:
 };
 
 
-void Context::Init(int width, int height, const std::string& name)
+void Context::Init(int width, int height, const std::string& name,
+			bool resizable,
+			bool visible,
+			bool decorated,
+			bool focused,
+			bool auto_iconify,
+			bool floating,
+			bool maximized,
+			bool transperent,
+			bool focus_on_show,
+			bool scale_to_monitor,
+			bool full_screen)
 {
 	if (m_window)
 	{
@@ -76,6 +101,17 @@ void Context::Init(int width, int height, const std::string& name)
 				"GLFW initialization failed (glfwInit() failed).\nThis may happen if you try to run bimpy on a headless machine ");
 	}
 
+	glfwWindowHint(GLFW_RESIZABLE, (int)resizable);
+	glfwWindowHint(GLFW_VISIBLE, (int)visible);
+	glfwWindowHint(GLFW_DECORATED, (int)decorated);
+	glfwWindowHint(GLFW_FOCUSED, (int)focused);
+	glfwWindowHint(GLFW_AUTO_ICONIFY, (int)auto_iconify);
+	glfwWindowHint(GLFW_FLOATING, (int)floating);
+	glfwWindowHint(GLFW_MAXIMIZED, (int)maximized);
+	glfwWindowHint(GLFW_TRANSPARENT_FRAMEBUFFER, (int)transperent);
+	glfwWindowHint(GLFW_FOCUS_ON_SHOW, (int)focus_on_show);
+	glfwWindowHint(GLFW_SCALE_TO_MONITOR, (int)scale_to_monitor);
+
 #if __APPLE__
 	// GL 3.2 + GLSL 150
 	const char* glsl_version = "#version 150";
@@ -92,7 +128,7 @@ void Context::Init(int width, int height, const std::string& name)
 	//glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);            // 3.0+ only
 #endif
 
-	m_window = glfwCreateWindow(width, height, name.c_str(), NULL, NULL);
+	m_window = glfwCreateWindow(width, height, name.c_str(), full_screen ? glfwGetPrimaryMonitor() : NULL, NULL);
 	if (!m_window)
 	{
 		glfwTerminate();
@@ -120,7 +156,7 @@ void Context::Init(int width, int height, const std::string& name)
 	m_width = width;
 	m_height = height;
 
-	glfwSetWindowUserPointer(m_window, this); // replaced m_imp.get()
+	glfwSetWindowUserPointer(m_window, this);
 
 	glfwSetWindowSizeCallback(m_window, [](GLFWwindow* window, int width, int height)
 	{
@@ -128,8 +164,10 @@ void Context::Init(int width, int height, const std::string& name)
 		ctx->Resize(width, height);
 	});
 
-	glfwSetKeyCallback(m_window, [](GLFWwindow*, int key, int, int action, int mods)
+	glfwSetKeyCallback(m_window, [](GLFWwindow* window, int key, int, int action, int mods)
 	{
+		Context* ctx = static_cast<Context*>(glfwGetWindowUserPointer(window));
+
 		ImGuiIO& io = ImGui::GetIO();
 		if (action == GLFW_PRESS)
 		{
@@ -144,6 +182,10 @@ void Context::Init(int width, int height, const std::string& name)
 		io.KeyShift = io.KeysDown[GLFW_KEY_LEFT_SHIFT] || io.KeysDown[GLFW_KEY_RIGHT_SHIFT];
 		io.KeyAlt = io.KeysDown[GLFW_KEY_LEFT_ALT] || io.KeysDown[GLFW_KEY_RIGHT_ALT];
 		io.KeySuper = io.KeysDown[GLFW_KEY_LEFT_SUPER] || io.KeysDown[GLFW_KEY_RIGHT_SUPER];
+		if (ctx->keyboard_callback)
+		{
+			ctx->keyboard_callback(key, action, mods);
+		}
 	});
 
 	glfwSetCharCallback(m_window, [](GLFWwindow*, unsigned int c)
@@ -158,14 +200,30 @@ void Context::Init(int width, int height, const std::string& name)
 		io.MouseWheel += (float) yoffset * 2.0f;
 	});
 
-	glfwSetMouseButtonCallback(m_window, [](GLFWwindow*, int button, int action, int /*mods*/)
+	glfwSetMouseButtonCallback(m_window, [](GLFWwindow* window, int button, int action, int /*mods*/)
 	{
+		Context* ctx = static_cast<Context*>(glfwGetWindowUserPointer(window));
 		ImGuiIO& io = ImGui::GetIO();
 
 		if (button >= 0 && button < 3)
 		{
 			io.MouseDown[button] = action == GLFW_PRESS;
 		}
+		if (ctx->mouse_button_callback)
+		{
+			double x, y;
+			glfwGetCursorPos(window, &x, &y);
+			ctx->mouse_button_callback(button, action == GLFW_PRESS, (float)x, (float)y);
+		}
+	});
+
+	glfwSetCursorPosCallback(m_window, [](GLFWwindow* window, double x, double y)
+	{
+		Context* ctx = static_cast<Context*>(glfwGetWindowUserPointer(window));
+		if (ctx->mouse_position_callback)
+		{
+			ctx->mouse_position_callback((float)x, (float)y);
+		}
 	});
 }
 
@@ -219,6 +277,20 @@ void Context::NewFrame()
 	ImGui_ImplOpenGL3_NewFrame();
 	ImGui_ImplGlfw_NewFrame();
 	ImGui::NewFrame();
+
+	// Overwrite default window behaviour
+	if (GImGui->WithinFrameScopeWithImplicitWindow)
+	{
+        GImGui->CurrentWindow->Active = false;
+		GImGui->WithinFrameScopeWithImplicitWindow = false;
+		ImGui::End();
+		GImGui->WithinFrameScopeWithImplicitWindow = true;
+		ImGui::SetNextWindowPos(ImVec2(0, 0));
+		ImGui::SetNextWindowSize(ImGui::GetIO().DisplaySize);
+		ImGui::PushStyleVar(ImGuiStyleVar_WindowRounding, 0);
+		ImGui::Begin("Debug##Default2", nullptr, ImGuiWindowFlags_NoDecoration | ImGuiWindowFlags_NoMove);
+		ImGui::PopStyleVar();
+	}
 }
 
 
@@ -384,6 +456,8 @@ private:
 	GLuint m_textureHandle;
 };
 
+std::vector<unsigned short> range_to_list(const ImWchar* r) { std::vector<unsigned short> l; while(*r != 0) { l.push_back(*r); ++r; } return l; }
+
 
 void  AddLine(const ImVec2& a, const ImVec2& b, ImU32 col, float thickness){ ImGui::GetWindowDrawList()->AddLine(a, b, col, thickness); }
 void  AddRect(const ImVec2& a, const ImVec2& b, ImU32 col, float rounding, int rounding_corners_flags, float thickness){ ImGui::GetWindowDrawList()->AddRect(a, b, col, rounding, rounding_corners_flags, thickness); }
@@ -402,6 +476,7 @@ void  PathLineTo(const ImVec2& pos){ ImGui::GetWindowDrawList()->PathLineTo(pos)
 void  PathFillConvex(ImU32 col){ ImGui::GetWindowDrawList()->PathFillConvex(col); }
 void  PathStroke(ImU32 col, bool closed, float thickness){ ImGui::GetWindowDrawList()->PathStroke(col, closed, thickness); }
 
+
 PYBIND11_MODULE(_bimpy, m) {
 	static Bool null;
 	null.null = true;
@@ -436,7 +511,7 @@ PYBIND11_MODULE(_bimpy, m) {
 		.value("NoNavInputs", ImGuiWindowFlags_::ImGuiWindowFlags_NoNavInputs)
 		.value("NoNavFocus", ImGuiWindowFlags_::ImGuiWindowFlags_NoNavFocus)
 		.value("UnsavedDocument", ImGuiWindowFlags_::ImGuiWindowFlags_UnsavedDocument)
-		.value("NoNav ", ImGuiWindowFlags_::ImGuiWindowFlags_NoNav )
+		.value("NoNav", ImGuiWindowFlags_::ImGuiWindowFlags_NoNav )
 		.value("NoDecoration", ImGuiWindowFlags_::ImGuiWindowFlags_NoDecoration)
 		.value("NoInputs", ImGuiWindowFlags_::ImGuiWindowFlags_NoInputs)
 		.export_values();
@@ -586,9 +661,37 @@ PYBIND11_MODULE(_bimpy, m) {
 		.value("CollapsingHeader", ImGuiTreeNodeFlags_CollapsingHeader)
 		.export_values();
 
+
+	/*
+	 * int width, int height, const std::string& name,
+			bool resizable,
+			bool visible,
+			bool decorated,
+			bool focused,
+			bool auto_iconify,
+			bool floating,
+			bool maximized,
+			bool transperent,
+			bool focus_on_show,
+			bool scale_to_monitor,
+			bool full_screen)
+	 */
 	py::class_<Context>(m, "Context")
 		.def(py::init())
-		.def("init", &Context::Init, "Initializes context and creates window")
+		.def("init", &Context::Init, "Initializes context and creates window",
+				py::arg("width"), py::arg("height"), py::arg("name"),
+				py::arg("resizable") = true,
+				py::arg("visible") = true,
+				py::arg("decorated") = true,
+				py::arg("focused") = true,
+				py::arg("auto_iconify") = true,
+				py::arg("floating") = false,
+				py::arg("maximized") = false,
+				py::arg("transperent") = false,
+				py::arg("focus_on_show") = true,
+				py::arg("scale_to_monitor") = false,
+				py::arg("full_screen") = false
+		)
 		.def("new_frame", &Context::NewFrame, "Starts a new frame. NewFrame must be called before any imgui functions")
 		.def("render", &Context::Render, "Finalizes the frame and draws all UI. Render must be called after all imgui functions")
 		.def("should_close", &Context::ShouldClose)
@@ -596,6 +699,10 @@ PYBIND11_MODULE(_bimpy, m) {
 		.def_property_readonly("width", &Context::GetWidth)
 		.def_property_readonly("height", &Context::GetHeight)
 
+		.def_readwrite("mouse_button_callback", &Context::mouse_button_callback)
+		.def_readwrite("mouse_position_callback", &Context::mouse_position_callback)
+		.def_readwrite("keyboard_callback", &Context::keyboard_callback)
+
 		.def_readwrite("clear_color", &Context::clearColor)
 		.def("__enter__", &Context::NewFrame)
 		.def("__exit__", [](Context& self, py::object, py::object, py::object)
@@ -717,6 +824,21 @@ PYBIND11_MODULE(_bimpy, m) {
 				self.Colors[(int)a] = c;
 			});
 
+	py::class_<ImGuiIO>(m, "IO")
+		.def(py::init())
+			.def_readonly("display_size", &ImGuiIO::DisplaySize)
+			.def_readonly("delta_time", &ImGuiIO::DeltaTime)
+			.def_readwrite("font_global_scale", &ImGuiIO::FontGlobalScale)
+			.def_readonly("backend_platform_name", &ImGuiIO::BackendPlatformName)
+			.def_readonly("backend_render_name", &ImGuiIO::BackendRendererName)
+			.def_readonly("mouse_pos", &ImGuiIO::MousePos)
+			.def_readonly("mouse_down", &ImGuiIO::MouseDown)
+			.def_readonly("mouse_wheel", &ImGuiIO::MouseWheel)
+			.def_readonly("mouse_wheel", &ImGuiIO::MouseWheel)
+			;
+
+	m.def("getio", [](){ return &ImGui::GetIO(); }, py::return_value_policy::reference);
+
 	m.def("get_style", &ImGui::GetStyle);
 	m.def("set_style", [](const ImGuiStyle& a)
 		{
@@ -966,7 +1088,9 @@ PYBIND11_MODULE(_bimpy, m) {
 
 	m.def("button", &ImGui::Button, py::arg("label"), py::arg("size") = ImVec2(0,0));
 	m.def("small_button", &ImGui::SmallButton);
-	m.def("invisible_button", &ImGui::InvisibleButton);
+	m.def("invisible_button", &ImGui::InvisibleButton, "button behavior without the visuals, frequently useful to build custom behaviors using the public api (along with IsItemActive, IsItemHovered, etc.)");
+	m.def("arrow_button", &ImGui::ArrowButton, "square button with an arrow shape");
+
 	m.def("tree_node", [](const char* label){ return ImGui::TreeNode(label); }, py::arg("label"));
 	m.def("tree_node_ex", [](int id, ImGuiTreeNodeFlags flags, const char* label){ return ImGui::TreeNodeEx((void*)(intptr_t)id, flags, "%s", label); }, py::arg("id"), py::arg("flags") = 0, py::arg("label")="");
 	m.def("tree_pop", &ImGui::TreePop);
@@ -976,6 +1100,7 @@ PYBIND11_MODULE(_bimpy, m) {
 	m.def("collapsing_header", [](const char* label, ImGuiTreeNodeFlags flags){ return ImGui::CollapsingHeader(label, flags); }, py::arg("label"), py::arg("flags") = 0);
 	m.def("checkbox", [](const char* label, Bool& v){ return ImGui::Checkbox(label, &v.value); });
 	m.def("radio_button", [](const char* label, bool active){ return ImGui::RadioButton(label, active); });
+	m.def("radio_button", [](const char* label, Int& v, int v_button){ return ImGui::RadioButton(label, &v.value, v_button); });
 
 	m.def("begin_combo", &ImGui::BeginCombo, py::arg("label"), py::arg("preview_value"), py::arg("flags") = 0);
 	m.def("end_combo", &ImGui::EndCombo, "only call EndCombo() if BeginCombo() returns true!");
@@ -1486,14 +1611,55 @@ PYBIND11_MODULE(_bimpy, m) {
 	m.def("path_fill_convex", &PathFillConvex, py::arg("col"));
 	m.def("path_stroke", &PathStroke, py::arg("col"), py::arg("closed"), py::arg("thickness"));
 
-	m.def("add_font_from_file_ttf", [](
+	m.def("get_glyph_ranges_default", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesDefault()); });
+	m.def("get_glyph_ranges_korean", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesKorean()); });
+	m.def("get_glyph_ranges_japanese", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesJapanese()); });
+	m.def("get_glyph_ranges_chinese_full", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesChineseFull()); });
+	m.def("get_glyph_ranges_chinese_simplified_common", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesChineseSimplifiedCommon()); });
+	m.def("get_glyph_ranges_cyrillic", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesCyrillic()); });
+	m.def("get_glyph_ranges_thai", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesThai()); });
+	m.def("get_glyph_ranges_vietnamese", [](){ return range_to_list(ImGui::GetIO().Fonts->GetGlyphRangesVietnamese()); });
+
+	auto get_persistent_storage = []()->std::vector<std::shared_ptr<std::vector<unsigned short> > >&
+			{ static std::vector<std::shared_ptr<std::vector<unsigned short> > > x; return x;};
+
+	m.def("add_font_from_file_ttf", [get_persistent_storage](
 		std::string filename,
-		float size_pixels = 32.0f)
+		float size_pixels = 32.0f, int oversample=2, bool merge=false, std::vector<unsigned short> range={})
 		{
-			return ImGui::GetIO().Fonts->AddFontFromFileTTF(filename.c_str(), size_pixels);
+            ImFontConfig font_cfg = ImFontConfig();
+            font_cfg.OversampleH = oversample;
+            font_cfg.MergeMode = merge;
+			if (range.size() > 0)
+			{
+				// Data needs to be persistent in memory until the atlas is built
+				auto prange = std::make_shared<std::vector<unsigned short> >(range);
+				prange->push_back(0);
+				get_persistent_storage().push_back(prange);
+				return ImGui::GetIO().Fonts->AddFontFromFileTTF(filename.c_str(), size_pixels, &font_cfg, prange->data());
+			}
+			else
+				return ImGui::GetIO().Fonts->AddFontFromFileTTF(filename.c_str(), size_pixels, &font_cfg);
 		}
 		, py::arg("filename")
-		, py::arg("size_pixels"), py::return_value_policy::reference);
+		, py::arg("size_pixels"), py::arg("oversample") = 2
+		, py::arg("merge") = false, py::arg("range") = std::vector<int>(), py::return_value_policy::reference);
+
+	m.def("add_font_default", [get_persistent_storage](int oversample=2, bool merge=false)
+		{
+            ImFontConfig font_cfg = ImFontConfig();
+            font_cfg.OversampleH = oversample;
+            font_cfg.MergeMode = merge;
+			return ImGui::GetIO().Fonts->AddFontDefault(&font_cfg);
+		}
+		,py::arg("oversample") = 2
+		, py::arg("merge") = false, py::return_value_policy::reference);
+
+	m.def("build_font", [get_persistent_storage]()
+		{
+			ImGui::GetIO().Fonts->Build();
+			get_persistent_storage().clear();
+		});
 
 	m.def("push_font", &ImGui::PushFont);
 	m.def("pop_font", &ImGui::PopFont);
